@@ -25,12 +25,12 @@ from rich.style import Style
 from . import errors
 from ._cells import cell_len
 from ._loop import loop_last
-from .strip import Strip
-from ._typing import TypeAlias
 from .geometry import NULL_OFFSET, Offset, Region, Size
-
+from .strip import Strip
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from .widget import Widget
 
 
