@@ -1,16 +1,17 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from typing import ClassVar, NamedTuple, TYPE_CHECKING
+from typing import TYPE_CHECKING, ClassVar, NamedTuple
 
 from .geometry import Region, Size, Spacing
-from ._typing import TypeAlias
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from .widget import Widget
 
 ArrangeResult: TypeAlias = "tuple[list[WidgetPlacement], set[Widget]]"
-DockArrangeResult: TypeAlias = "tuple[list[WidgetPlacement], set[Widget], Spacing]]"
+DockArrangeResult: TypeAlias = "tuple[list[WidgetPlacement], set[Widget], Spacing]"
 
 
 class WidgetPlacement(NamedTuple):
@@ -56,7 +57,7 @@ class Layout(ABC):
         Returns:
             Width of the content.
         """
-        if not widget.children:
+        if not widget._nodes:
             width = 0
         else:
             # Use a size of 0, 0 to ignore relative sizes, since those are flexible anyway
@@ -84,7 +85,7 @@ class Layout(ABC):
         Returns:
             Content height (in lines).
         """
-        if not widget.children:
+        if not widget._nodes:
             height = 0
         else:
             # Use a height of zero to ignore relative heights
