@@ -17,7 +17,7 @@ from weakref import WeakSet
 from . import Logger, events, log, messages
 from ._asyncio import create_task
 from ._callback import invoke
-from ._context import NoActiveAppError, active_app
+from ._context import NoActiveAppError, active_app, active_message_pump
 from ._time import time
 from .case import camel_to_snake
 from .errors import DuplicateKeyHandlers
@@ -64,6 +64,7 @@ class MessagePumpMeta(type):
 class MessagePump(metaclass=MessagePumpMeta):
     def __init__(self, parent: MessagePump | None = None) -> None:
         self._message_queue: Queue[Message | None] = Queue()
+        self._active_message: Message | None = None
         self._parent = parent
         self._running: bool = False
         self._closing: bool = False
@@ -75,6 +76,7 @@ class MessagePump(metaclass=MessagePumpMeta):
         self._last_idle: float = time()
         self._max_idle: float | None = None
         self._mounted_event = asyncio.Event()
+        self._next_callbacks: list[CallbackType] = []
 
     @property
     def task(self) -> Task:
@@ -121,6 +123,19 @@ class MessagePump(metaclass=MessagePumpMeta):
         """
         return self.app._logger
 
+    @property
+    def is_attached(self) -> bool:
+        """Is the node is attached to the app via the DOM."""
+        from .app import App
+
+        node = self
+
+        while not isinstance(node, App):
+            if node._parent is None:
+                return False
+            node = node._parent
+        return True
+
     def _attach(self, parent: MessagePump) -> None:
         """Set the parent, and therefore attach this node to the tree.
 
@@ -271,10 +286,22 @@ class MessagePump(metaclass=MessagePumpMeta):
 
         Args:
             callback: Callable to call next.
+            *args: Positional arguments to pass to the callable.
+            **kwargs: Keyword arguments to pass to the callable.
         """
         message = events.Callback(self, callback=partial(callback, *args, **kwargs))
         self.post_message_no_wait(message)
 
+    def call_next(self, callback: Callable, *args, **kwargs) -> None:
+        """Schedule a callback to run immediately after processing the current message.
+
+        Args:
+            callback: Callable to run after current event.
+            *args: Positional arguments to pass to the callable.
+            **kwargs: Keyword arguments to pass to the callable.
+        """
+        self._next_callbacks.append(partial(callback, *args, **kwargs))
+
     def _on_invoke_later(self, message: messages.InvokeLater) -> None:
         # Forward InvokeLater message to the Screen
         self.app.screen._invoke_later(message.callback)
@@ -299,12 +326,18 @@ class MessagePump(metaclass=MessagePumpMeta):
         Reactive._reset_object(self)
         await self._message_queue.put(None)
         if wait and self._task is not None and asyncio.current_task() != self._task:
-            # Ensure everything is closed before returning
-            await self._task
+            try:
+                running_widget = active_message_pump.get()
+            except LookupError:
+                running_widget = None
+
+            if running_widget is None or running_widget is not self:
+                await self._task
 
     def _start_messages(self) -> None:
         """Start messages task."""
         if self.app._running:
+            active_message_pump.set(self)
             self._task = create_task(
                 self._process_messages(), name=f"message pump {self}"
             )
@@ -338,12 +371,14 @@ class MessagePump(metaclass=MessagePumpMeta):
         finally:
             # This is critical, mount may be waiting
             self._mounted_event.set()
-        Reactive._initialize_object(self)
+        self._post_mount()
+
+    def _post_mount(self):
+        """Called after the object has been mounted."""
 
     async def _process_messages_loop(self) -> None:
         """Process messages until the queue is closed."""
         _rich_traceback_guard = True
-
         while not self._closed:
             try:
                 message = await self._get_message()
@@ -367,35 +402,51 @@ class MessagePump(metaclass=MessagePumpMeta):
                 except MessagePumpClosed:
                     break
 
+            self._active_message = message
+
             try:
-                await self._dispatch_message(message)
-            except CancelledError:
-                raise
+                try:
+                    await self._dispatch_message(message)
+                except CancelledError:
+                    raise
+                except Exception as error:
+                    self._mounted_event.set()
+                    self.app._handle_exception(error)
+                    break
+                finally:
+                    self._message_queue.task_done()
+
+                    current_time = time()
+
+                    # Insert idle events
+                    if self._message_queue.empty() or (
+                        self._max_idle is not None
+                        and current_time - self._last_idle > self._max_idle
+                    ):
+                        self._last_idle = current_time
+                        if not self._closed:
+                            event = events.Idle(self)
+                            for _cls, method in self._get_dispatch_methods(
+                                "on_idle", event
+                            ):
+                                try:
+                                    await invoke(method, event)
+                                except Exception as error:
+                                    self.app._handle_exception(error)
+                                    break
+            finally:
+                self._active_message = None
+
+    async def _flush_next_callbacks(self) -> None:
+        """Invoke pending callbacks in next callbacks queue."""
+        callbacks = self._next_callbacks.copy()
+        self._next_callbacks.clear()
+        for callback in callbacks:
+            try:
+                await invoke(callback)
             except Exception as error:
-                self._mounted_event.set()
                 self.app._handle_exception(error)
                 break
-            finally:
-
-                self._message_queue.task_done()
-                current_time = time()
-
-                # Insert idle events
-                if self._message_queue.empty() or (
-                    self._max_idle is not None
-                    and current_time - self._last_idle > self._max_idle
-                ):
-                    self._last_idle = current_time
-                    if not self._closed:
-                        event = events.Idle(self)
-                        for _cls, method in self._get_dispatch_methods(
-                            "on_idle", event
-                        ):
-                            try:
-                                await invoke(method, event)
-                            except Exception as error:
-                                self.app._handle_exception(error)
-                                break
 
     async def _dispatch_message(self, message: Message) -> None:
         """Dispatch a message received from the message queue.
@@ -412,6 +463,7 @@ class MessagePump(metaclass=MessagePumpMeta):
             await self.on_event(message)
         else:
             await self._on_message(message)
+        await self._flush_next_callbacks()
 
     def _get_dispatch_methods(
         self, method_name: str, message: Message
@@ -544,34 +596,6 @@ class MessagePump(metaclass=MessagePumpMeta):
     async def on_callback(self, event: events.Callback) -> None:
         await invoke(event.callback)
 
-    def emit_no_wait(self, message: Message) -> bool:
-        """Send a message to the _parent_, non async version.
-
-        Args:
-            message: A message object.
-
-        Returns:
-            True if the message was posted successfully.
-        """
-        if self._parent:
-            return self._parent._post_message_from_child_no_wait(message)
-        else:
-            return False
-
-    async def emit(self, message: Message) -> bool:
-        """Send a message to the _parent_.
-
-        Args:
-            message: A message object.
-
-        Returns:
-            True if the message was posted successfully.
-        """
-        if self._parent:
-            return await self._parent._post_message_from_child(message)
-        else:
-            return False
-
     # TODO: Does dispatch_key belong on message pump?
     async def dispatch_key(self, event: events.Key) -> bool:
         """Dispatch a key event to method.
