@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import Iterable, Iterator
+from typing import TYPE_CHECKING, Iterable, Iterator
 
 import rich.repr
 from rich.console import RenderableType
@@ -9,17 +9,18 @@ from rich.style import Style
 from . import errors, events, messages
 from ._callback import invoke
 from ._compositor import Compositor, MapGeometry
+from ._types import CallbackType
 from .css.match import match
 from .css.parse import parse_selectors
 from .dom import DOMNode
-from .timer import Timer
-from ._types import CallbackType
 from .geometry import Offset, Region, Size
-from ._typing import Final
 from .reactive import Reactive
 from .renderables.blank import Blank
+from .timer import Timer
 from .widget import Widget
 
+if TYPE_CHECKING:
+    from typing_extensions import Final
 
 # Screen updates will be batched so that they don't happen more often than 120 times per second:
 UPDATE_PERIOD: Final[float] = 1 / 120
@@ -62,7 +63,7 @@ class Screen(Widget):
 
     @property
     def is_current(self) -> bool:
-        """Check if this screen is current (i.e. visible to user)."""
+        """Is the screen current (i.e. visible to user)?"""
         from .app import ScreenStackError
 
         try:
@@ -230,7 +231,7 @@ class Screen(Widget):
         else:
             # Only move the focus if we are currently showing the focus
             if direction:
-                to_focus: Widget | None = None
+                to_focus = None
                 chain_length = len(focus_chain)
                 for step in range(1, len(focus_chain) + 1):
                     node = focus_chain[
@@ -367,19 +368,20 @@ class Screen(Widget):
         # Check for any widgets marked as 'dirty' (needs a repaint)
         event.prevent_default()
 
-        async with self.app._dom_lock:
-            if self.is_current:
-                if self._layout_required:
-                    self._refresh_layout()
-                    self._layout_required = False
-                    self._dirty_widgets.clear()
-                if self._repaint_required:
-                    self._dirty_widgets.clear()
-                    self._dirty_widgets.add(self)
-                    self._repaint_required = False
+        if self.is_current:
+            async with self.app._dom_lock:
+                if self.is_current:
+                    if self._layout_required:
+                        self._refresh_layout()
+                        self._layout_required = False
+                        self._dirty_widgets.clear()
+                    if self._repaint_required:
+                        self._dirty_widgets.clear()
+                        self._dirty_widgets.add(self)
+                        self._repaint_required = False
 
-                if self._dirty_widgets:
-                    self.update_timer.resume()
+                    if self._dirty_widgets:
+                        self.update_timer.resume()
 
         # The Screen is idle - a good opportunity to invoke the scheduled callbacks
         await self._invoke_and_clear_callbacks()
