@@ -30,14 +30,14 @@ class ScrollView(Widget):
         """Not transparent, i.e. renders something."""
         return False
 
-    def watch_scroll_x(self, new_value: float) -> None:
-        if self.show_horizontal_scrollbar:
-            self.horizontal_scrollbar.position = int(new_value)
+    def watch_scroll_x(self, old_value: float, new_value: float) -> None:
+        if self.show_horizontal_scrollbar and round(old_value) != round(new_value):
+            self.horizontal_scrollbar.position = round(new_value)
             self.refresh()
 
-    def watch_scroll_y(self, new_value: float) -> None:
-        if self.show_vertical_scrollbar:
-            self.vertical_scrollbar.position = int(new_value)
+    def watch_scroll_y(self, old_value: float, new_value: float) -> None:
+        if self.show_vertical_scrollbar and round(old_value) != round(new_value):
+            self.vertical_scrollbar.position = round(new_value)
             self.refresh()
 
     def on_mount(self):
