@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from collections import deque
-from typing import Iterable, Iterator, TypeVar, overload, TYPE_CHECKING
+from typing import TYPE_CHECKING, Iterable, Iterator, TypeVar, overload
 
 if TYPE_CHECKING:
     from textual.dom import DOMNode
@@ -53,7 +53,7 @@ def walk_depth_first(
     """
     from textual.dom import DOMNode
 
-    stack: list[Iterator[DOMNode]] = [iter(root.children)]
+    stack: list[Iterator[DOMNode]] = [iter(root._nodes)]
     pop = stack.pop
     push = stack.append
     check_type = filter_type or DOMNode
