@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 from asyncio import Lock, wait
-from asyncio import Lock, create_task, wait
 from collections import Counter
 from fractions import Fraction
 from itertools import islice
@@ -36,9 +35,9 @@ from rich.text import Text
 from rich.traceback import Traceback
 
 from . import errors, events, messages
-from ._asyncio import create_task
 from ._animator import DEFAULT_EASING, Animatable, BoundAnimator, EasingFunction
 from ._arrange import DockArrangeResult, arrange
+from ._asyncio import create_task
 from ._context import active_app
 from ._easing import DEFAULT_SCROLL_EASING
 from ._layout import Layout
@@ -91,6 +90,10 @@ class AwaitMount:
         self._parent = parent
         self._widgets = widgets
 
+    async def __call__(self) -> None:
+        """Allows awaiting via a call operation."""
+        await self
+
     def __await__(self) -> Generator[None, None, None]:
         async def await_mount() -> None:
             if self._widgets:
@@ -114,7 +117,7 @@ class _Styled:
     """
 
     def __init__(
-        self, renderable: "RenderableType", style: Style, link_style: Style | None
+        self, renderable: "ConsoleRenderable", style: Style, link_style: Style | None
     ) -> None:
         self.renderable = renderable
         self.style = style
@@ -130,7 +133,7 @@ class _Styled:
         if style:
             apply = style.__add__
             result_segments = (
-                _Segment(text, apply(_style), control)
+                _Segment(text, apply(_style), None)
                 for text, _style, control in result_segments
             )
         link_style = self.link_style
@@ -138,19 +141,22 @@ class _Styled:
             result_segments = (
                 _Segment(
                     text,
-                    style
-                    if style._meta is None
-                    else (style + link_style if "@click" in style.meta else style),
+                    (
+                        style
+                        if style._meta is None
+                        else (style + link_style if "@click" in style.meta else style)
+                    ),
                     control,
                 )
                 for text, style, control in result_segments
+                if style is not None
             )
         return result_segments
 
     def __rich_measure__(
         self, console: "Console", options: "ConsoleOptions"
     ) -> Measurement:
-        return self.renderable.__rich_measure__(console, options)
+        return Measurement.get(console, options, self.renderable)
 
 
 class RenderCache(NamedTuple):
@@ -293,7 +299,7 @@ class Widget(DOMNode):
         """
         parent = self.parent
         if parent is not None:
-            siblings = list(parent.children)
+            siblings = list(parent._nodes)
             siblings.remove(self)
             return siblings
         else:
@@ -384,7 +390,7 @@ class Widget(DOMNode):
             NoMatches: if no children could be found for this ID
             WrongType: if the wrong type was found.
         """
-        child = self.children._get_by_id(id)
+        child = self._nodes._get_by_id(id)
         if child is None:
             raise NoMatches(f"No child found with id={id!r}")
         if expect_type is None:
@@ -466,7 +472,7 @@ class Widget(DOMNode):
         """
         assert self.is_container
 
-        cache_key = (size, self.children._updates)
+        cache_key = (size, self._nodes._updates)
         if (
             self._arrangement_cache_key == cache_key
             and self._cached_arrangement is not None
@@ -475,7 +481,7 @@ class Widget(DOMNode):
 
         self._arrangement_cache_key = cache_key
         arrangement = self._cached_arrangement = arrange(
-            self, self.children, size, self.screen.size
+            self, self._nodes, size, self.screen.size
         )
 
         return arrangement
@@ -543,7 +549,7 @@ class Widget(DOMNode):
         # children. We should be able to go looking for the widget's
         # location amongst its parent's children.
         try:
-            return cast("Widget", spot.parent), spot.parent.children.index(spot)
+            return cast("Widget", spot.parent), spot.parent._nodes.index(spot)
         except ValueError:
             raise MountError(f"{spot!r} is not a child of {self!r}") from None
 
@@ -557,8 +563,12 @@ class Widget(DOMNode):
 
         Args:
             *widgets: The widget(s) to mount.
-            before: Optional location to mount before.
-            after: Optional location to mount after.
+            before: Optional location to mount before. An `int` is the index
+                of the child to mount before, a `str` is a `query_one` query to
+                find the widget to mount before.
+            after: Optional location to mount after. An `int` is the index
+                of the child to mount after, a `str` is a `query_one` query to
+                find the widget to mount after.
 
         Returns:
             An awaitable object that waits for widgets to be mounted.
@@ -606,7 +616,40 @@ class Widget(DOMNode):
             parent, *widgets, before=insert_before, after=insert_after
         )
 
-        return AwaitMount(self, mounted)
+        await_mount = AwaitMount(self, mounted)
+        self.call_next(await_mount)
+        return await_mount
+
+    def mount_all(
+        self,
+        widgets: Iterable[Widget],
+        *,
+        before: int | str | Widget | None = None,
+        after: int | str | Widget | None = None,
+    ) -> AwaitMount:
+        """Mount widgets from an iterable.
+
+        Args:
+            widgets: An iterable of widgets.
+            before: Optional location to mount before. An `int` is the index
+                of the child to mount before, a `str` is a `query_one` query to
+                find the widget to mount before.
+            after: Optional location to mount after. An `int` is the index
+                of the child to mount after, a `str` is a `query_one` query to
+                find the widget to mount after.
+
+        Returns:
+            An awaitable object that waits for widgets to be mounted.
+
+        Raises:
+            MountError: If there is a problem with the mount request.
+
+        Note:
+            Only one of ``before`` or ``after`` can be provided. If both are
+            provided a ``MountError`` will be raised.
+        """
+        await_mount = self.mount(*widgets, before=before, after=after)
+        return await_mount
 
     def move_child(
         self,
@@ -618,8 +661,12 @@ class Widget(DOMNode):
 
         Args:
             child: The child widget to move.
-            before: (int | Widget, optional): Optional location to move before.
-            after: (int | Widget, optional): Optional location to move after.
+            before: Optional location to move before. An `int` is the index
+                of the child to move before, a `str` is a `query_one` query to
+                find the widget to move before.
+            after: Optional location to move after. An `int` is the index
+                of the child to move after, a `str` is a `query_one` query to
+                find the widget to move after.
 
         Raises:
             WidgetError: If there is a problem with the child or target.
@@ -640,7 +687,7 @@ class Widget(DOMNode):
             """Ensure a given child reference is a Widget."""
             if isinstance(child, int):
                 try:
-                    child = self.children[child]
+                    child = self._nodes[child]
                 except IndexError:
                     raise WidgetError(
                         f"An index of {child} for the child to {called} is out of bounds"
@@ -649,7 +696,7 @@ class Widget(DOMNode):
                 # We got an actual widget, so let's be sure it really is one of
                 # our children.
                 try:
-                    _ = self.children.index(child)
+                    _ = self._nodes.index(child)
                 except ValueError:
                     raise WidgetError(f"{child!r} is not a child of {self!r}") from None
             return child
@@ -662,11 +709,11 @@ class Widget(DOMNode):
         # child; where we're moving it to, which should be within the child
         # list; and how we're supposed to move it. All that's left is doing
         # the right thing.
-        self.children._remove(child)
+        self._nodes._remove(child)
         if before is not None:
-            self.children._insert(self.children.index(target), child)
+            self._nodes._insert(self._nodes.index(target), child)
         else:
-            self.children._insert(self.children.index(target) + 1, child)
+            self._nodes._insert(self._nodes.index(target) + 1, child)
 
         # Request a refresh.
         self.refresh(layout=True)
@@ -751,7 +798,9 @@ class Widget(DOMNode):
         console = self.app.console
         renderable = self._render()
 
-        width = measure(console, renderable, container.width)
+        width = measure(
+            console, renderable, container.width, container_width=container.width
+        )
         if self.expand:
             width = max(container.width, width)
         if self.shrink:
@@ -908,7 +957,7 @@ class Widget(DOMNode):
         show_horizontal = self.show_horizontal_scrollbar
         if overflow_x == "hidden":
             show_horizontal = False
-        if overflow_x == "scroll":
+        elif overflow_x == "scroll":
             show_horizontal = True
         elif overflow_x == "auto":
             show_horizontal = self.virtual_size.width > width
@@ -1132,9 +1181,7 @@ class Widget(DOMNode):
             List of widgets that can receive focus.
 
         """
-        focusable = [
-            child for child in self.children if child.display and child.visible
-        ]
+        focusable = [child for child in self._nodes if child.display and child.visible]
         return sorted(focusable, key=attrgetter("_focus_sort_key"))
 
     @property
@@ -1228,7 +1275,7 @@ class Widget(DOMNode):
         Returns:
             True if this widget is a container.
         """
-        return self.styles.layout is not None or bool(self.children)
+        return self.styles.layout is not None or bool(self._nodes)
 
     @property
     def is_scrollable(self) -> bool:
@@ -1237,7 +1284,7 @@ class Widget(DOMNode):
         Returns:
             True if this widget may be scrolled.
         """
-        return self.styles.layout is not None or bool(self.children)
+        return self.styles.layout is not None or bool(self._nodes)
 
     @property
     def layer(self) -> str:
@@ -1368,6 +1415,7 @@ class Widget(DOMNode):
                 easing = DEFAULT_SCROLL_EASING
 
             if maybe_scroll_x:
+                assert x is not None
                 self.scroll_target_x = x
                 if x != self.scroll_x:
                     self.animate(
@@ -1379,6 +1427,7 @@ class Widget(DOMNode):
                     )
                     scrolled_x = True
             if maybe_scroll_y:
+                assert y is not None
                 self.scroll_target_y = y
                 if y != self.scroll_y:
                     self.animate(
@@ -1392,10 +1441,12 @@ class Widget(DOMNode):
 
         else:
             if maybe_scroll_x:
+                assert x is not None
                 scroll_x = self.scroll_x
                 self.scroll_target_x = self.scroll_x = x
                 scrolled_x = scroll_x != self.scroll_x
             if maybe_scroll_y:
+                assert y is not None
                 scroll_y = self.scroll_y
                 self.scroll_target_y = self.scroll_y = y
                 scrolled_y = scroll_y != self.scroll_y
@@ -1531,7 +1582,7 @@ class Widget(DOMNode):
 
         """
         return self.scroll_to(
-            x=self.scroll_target_x - 1,
+            x=self.scroll_target_x - self.app.scroll_sensitivity_x,
             animate=animate,
             speed=speed,
             duration=duration,
@@ -1563,7 +1614,7 @@ class Widget(DOMNode):
 
         """
         return self.scroll_to(
-            x=self.scroll_target_x + 1,
+            x=self.scroll_target_x + self.app.scroll_sensitivity_x,
             animate=animate,
             speed=speed,
             duration=duration,
@@ -1595,7 +1646,7 @@ class Widget(DOMNode):
 
         """
         return self.scroll_to(
-            y=self.scroll_target_y + 1,
+            y=self.scroll_target_y + self.app.scroll_sensitivity_y,
             animate=animate,
             speed=speed,
             duration=duration,
@@ -1627,7 +1678,7 @@ class Widget(DOMNode):
 
         """
         return self.scroll_to(
-            y=self.scroll_target_y - 1,
+            y=self.scroll_target_y - +self.app.scroll_sensitivity_y,
             animate=animate,
             speed=speed,
             duration=duration,
@@ -1960,13 +2011,14 @@ class Widget(DOMNode):
         """
         show_vertical_scrollbar, show_horizontal_scrollbar = self.scrollbars_enabled
 
-        scrollbar_size_horizontal = self.styles.scrollbar_size_horizontal
-        scrollbar_size_vertical = self.styles.scrollbar_size_vertical
+        styles = self.styles
+        scrollbar_size_horizontal = styles.scrollbar_size_horizontal
+        scrollbar_size_vertical = styles.scrollbar_size_vertical
 
-        if self.styles.scrollbar_gutter == "stable":
+        if styles.scrollbar_gutter == "stable":
             # Let's _always_ reserve some space, whether the scrollbar is actually displayed or not:
             show_vertical_scrollbar = True
-            scrollbar_size_vertical = self.styles.scrollbar_size_vertical
+            scrollbar_size_vertical = styles.scrollbar_size_vertical
 
         if show_horizontal_scrollbar and show_vertical_scrollbar:
             (region, _, _, _) = region.split(
@@ -2418,12 +2470,12 @@ class Widget(DOMNode):
     def _on_focus(self, event: events.Focus) -> None:
         self.has_focus = True
         self.refresh()
-        self.emit_no_wait(events.DescendantFocus(self))
+        self.post_message_no_wait(events.DescendantFocus(self))
 
     def _on_blur(self, event: events.Blur) -> None:
         self.has_focus = False
         self.refresh()
-        self.emit_no_wait(events.DescendantBlur(self))
+        self.post_message_no_wait(events.DescendantBlur(self))
 
     def _on_descendant_blur(self, event: events.DescendantBlur) -> None:
         if self._has_focus_within:
@@ -2433,15 +2485,25 @@ class Widget(DOMNode):
         if self._has_focus_within:
             self.app.update_styles(self)
 
-    def _on_mouse_scroll_down(self, event) -> None:
-        if self.allow_vertical_scroll:
-            if self.scroll_down(animate=False):
-                event.stop()
+    def _on_mouse_scroll_down(self, event: events.MouseScrollDown) -> None:
+        if event.ctrl or event.shift:
+            if self.allow_horizontal_scroll:
+                if self.scroll_right(animate=False):
+                    event.stop()
+        else:
+            if self.allow_vertical_scroll:
+                if self.scroll_down(animate=False):
+                    event.stop()
 
-    def _on_mouse_scroll_up(self, event) -> None:
-        if self.allow_vertical_scroll:
-            if self.scroll_up(animate=False):
-                event.stop()
+    def _on_mouse_scroll_up(self, event: events.MouseScrollUp) -> None:
+        if event.ctrl or event.shift:
+            if self.allow_horizontal_scroll:
+                if self.scroll_left(animate=False):
+                    event.stop()
+        else:
+            if self.allow_vertical_scroll:
+                if self.scroll_up(animate=False):
+                    event.stop()
 
     def _on_scroll_to(self, message: ScrollTo) -> None:
         if self._allow_scroll:
