@@ -6,13 +6,13 @@ from typing import cast
 import rich.repr
 from rich.console import RenderableType
 from rich.text import Text, TextType
+from typing_extensions import Literal
 
 from .. import events
 from ..css._error_tools import friendly_list
 from ..message import Message
-from ..reactive import Reactive
+from ..reactive import reactive
 from ..widgets import Static
-from .._typing import Literal
 
 ButtonVariant = Literal["default", "primary", "success", "warning", "error"]
 _VALID_BUTTON_VARIANTS = {"default", "primary", "success", "warning", "error"}
@@ -150,9 +150,21 @@ class Button(Static, can_focus=True):
     ACTIVE_EFFECT_DURATION = 0.3
     """When buttons are clicked they get the `-active` class for this duration (in seconds)"""
 
+    label: reactive[RenderableType] = reactive("")
+    """The text label that appears within the button."""
+
+    variant = reactive("default")
+    """The variant name for the button."""
+
+    disabled = reactive(False)
+    """The disabled state of the button; `True` if disabled, `False` if not."""
+
     class Pressed(Message, bubble=True):
         """Event sent when a `Button` is pressed.
 
+        Can be handled using `on_button_pressed` in a subclass of `Button` or
+        in a parent widget in the DOM.
+
         Attributes:
             button: The button that was pressed.
         """
@@ -194,15 +206,6 @@ class Button(Static, can_focus=True):
 
         self.variant = self.validate_variant(variant)
 
-    label: Reactive[RenderableType] = Reactive("")
-    """The text label that appears within the button."""
-
-    variant = Reactive.init("default")
-    """The variant name for the button."""
-
-    disabled = Reactive(False)
-    """The disabled state of the button; `True` if disabled, `False` if not."""
-
     def __rich_repr__(self) -> rich.repr.Result:
         yield from super().__rich_repr__()
         yield "variant", self.variant, "default"
@@ -251,7 +254,7 @@ class Button(Static, can_focus=True):
         # Manage the "active" effect:
         self._start_active_affect()
         # ...and let other components know that we've just been clicked:
-        self.emit_no_wait(Button.Pressed(self))
+        self.post_message_no_wait(Button.Pressed(self))
 
     def _start_active_affect(self) -> None:
         """Start a small animation to show the button was clicked."""
@@ -263,7 +266,7 @@ class Button(Static, can_focus=True):
     async def _on_key(self, event: events.Key) -> None:
         if event.key == "enter" and not self.disabled:
             self._start_active_affect()
-            await self.emit(Button.Pressed(self))
+            await self.post_message(Button.Pressed(self))
 
     @classmethod
     def success(
