@@ -2,11 +2,12 @@ from __future__ import annotations
 
 from itertools import cycle
 
+from typing_extensions import Literal
+
 from .. import events
 from ..css._error_tools import friendly_list
 from ..reactive import Reactive, reactive
-from ..widget import Widget, RenderResult
-from .._typing import Literal
+from ..widget import RenderResult, Widget
 
 PlaceholderVariant = Literal["default", "size", "text"]
 _VALID_PLACEHOLDER_VARIANTS_ORDERED: list[PlaceholderVariant] = [
