@@ -10,11 +10,10 @@ from rich.protocol import is_renderable
 from rich.segment import Segment
 from rich.text import Text
 
-from ..reactive import var
-from ..geometry import Size, Region
-from ..scroll_view import ScrollView
 from .._cache import LRUCache
-from .._segment_tools import line_crop
+from ..geometry import Region, Size
+from ..reactive import var
+from ..scroll_view import ScrollView
 from ..strip import Strip
 
 
@@ -160,7 +159,6 @@ class TextLog(ScrollView, can_focus=True):
         return lines
 
     def _render_line(self, y: int, scroll_x: int, width: int) -> Strip:
-
         if y >= len(self.lines):
             return Strip.blank(width, self.rich_style)
 
